# Generates a regina file from a file of isomorphism signatures

from readDataFiles import parse_text

from regina import NTriangulation, NContainer

import os, sys

if __name__ == '__main__':
    # Check proper usage
    if not len(sys.argv) == 4:
        print >>sys.stderr, (
            "Usage: genReginaCensusFiles INPUT_FILE ROOT_ITEM_LABEL REGINA_FILE")
        sys.exit(1)

    # Parse command line
    inputFile, rootItemName, rgaFile = sys.argv[1:]

    # Read in file of isomorphism signatures
    namedListOfListOfIsoSigs = parse_text(open(inputFile).read())
    
    # Determine the number of tetrahedra occuring
    numbers_of_tetrahedra = set()
    for name, listOfIsoSigs, isometry_signature in namedListOfListOfIsoSigs:
        representative = listOfIsoSigs[0]
        numbers_of_tetrahedra.add(NTriangulation.isoSigComponentSize(representative))

    # Create root items
    rootItem = NContainer()
    rootItem.setPacketLabel(rootItemName)

    # Create a container for each number of tetrahedra
    tet_items = {}
    for number_of_tetrahedra in sorted(numbers_of_tetrahedra):
        tet_items[number_of_tetrahedra] = NContainer()
    
    # Iterate through the named lists of isomorphism signatures
    for name, listOfIsoSigs, isometry_signature in namedListOfListOfIsoSigs:

        # Create container for manifold
        manifold_item = NContainer()
        manifold_item.setPacketLabel(name)
        representative = listOfIsoSigs[0]
        number_of_tetrahedra = NTriangulation.isoSigComponentSize(representative)

        for i, isoSig in enumerate(listOfIsoSigs):
            # Create triangulations for each isomorphism signature
            tessellation_item = NTriangulation.fromIsoSig(isoSig)
            tessellation_item.setPacketLabel(
                '%s#%d' % (name, i))

            manifold_item.insertChildLast(tessellation_item)

        canon_item = NTriangulation.fromIsoSig(isometry_signature)
        canon_item.setPacketLabel(
            '%s#Canonical' % name)
        manifold_item.insertChildLast(canon_item)

        tet_items[number_of_tetrahedra].insertChildLast(manifold_item)

    # Now put it all together
    for number_of_tetrahedra in sorted(numbers_of_tetrahedra):
        tet_items[number_of_tetrahedra].setPacketLabel(
            '%d Tetrahedra' % number_of_tetrahedra)
        rootItem.insertChildLast(tet_items[number_of_tetrahedra])

    # And save
    rootItem.save(rgaFile)
